package Classes 
{
  import flash.geom.*; 
  import flash.display.*; 

  public class Snake
  {
    public static const SPEED:int       = 4;  
    public static const NORTH:int       = 1;
    public static const EAST:int        = 2;
    public static const SOUTH:int       = 3;
    public static const WEST:int        = 4; 
  
    private var direction:int = NORTH;  
    private var newDirection:int = NORTH;    
    private var partPosition:Array = new Array();
    private var partDirection:Array = new Array();	
    private var partPhase:Array = new Array();	
  
    public function Snake(xPos:int,yPos:int,length:int) 
    {      
      for (var i:int = 0; i < length; i++)
      {
        partPosition.push(new Point(xPos,yPos - i * SPEED)); 
        partDirection.push(direction);
        partPhase.push(i % 9);
      }
    }
    
    public function getPosition():Point
    {
      return partPosition[0];
    }
    
    public function getDirection():int
    {
      return direction;
    }
    
    public function setDirection(direction:int):void
    {
      newDirection = direction;
    }
    
    public function move():void
    {    
      partPosition.pop();
      partDirection.pop();
      partPhase.pop();
      
      var position:Point = partPosition[0];
      var phase:int = partPhase[0];    
         
      switch (direction)
      {
      case NORTH:
        if ((newDirection != direction) && (phase == 0))      
        {                
          direction = newDirection;          
          
          if (direction == EAST)
            partPosition.unshift(new Point(position.x + 7,position.y + 8));
          else
            partPosition.unshift(new Point(position.x - 6,position.y + 8)); 
                       
          partPhase.unshift(0);  
        }
        else
        {
          partPosition.unshift(new Point(position.x,position.y + SPEED));             
          partPhase.unshift(((phase - 1) < 0 ? 8 : phase - 1)  % 9);
        }    
        break;
      case EAST:
        if ((newDirection != direction) && (phase == 0))      
        { 
          direction = newDirection;
          
          if (direction == NORTH)
            partPosition.unshift(new Point(position.x + 4,position.y + 5));
          else
            partPosition.unshift(new Point(position.x + 4,position.y - 8));        

          partPhase.unshift(0);  
        }
        else      
        {
          partPosition.unshift(new Point(position.x + SPEED,position.y));          
          partPhase.unshift((phase + 1) % 9); 
        }                  
        break;
      case SOUTH:
        if ((newDirection != direction) && (phase == 0))      
        {                
          direction = newDirection;          
          
          if (direction == EAST)
            partPosition.unshift(new Point(position.x + 8,position.y - 6));
          else
            partPosition.unshift(new Point(position.x - 4,position.y - 5)); 
                       
          partPhase.unshift(0);  
        }
        else
        {      
          partPosition.unshift(new Point(position.x,position.y - SPEED));     
          partPhase.unshift((phase + 1) % 9); 
        }        
        break;
      case WEST:
        if ((newDirection != direction) && (phase == 0))      
        { 
          direction = newDirection;
          
          if (direction == NORTH)
            partPosition.unshift(new Point(position.x - 7,position.y + 5));
          else
            partPosition.unshift(new Point(position.x - 7,position.y - 8));       
          
          partPhase.unshift(0);  
        }
        else      
        {      
          partPosition.unshift(new Point(position.x - SPEED,position.y));      
          partPhase.unshift(((phase - 1) < 0 ? 8 : phase - 1)  % 9); 
        }   
        break;
      }   
      
      partDirection.unshift(direction);         
    } 
    
    [Embed(source="../Images/Snake.png")] private var snakeImg:Class;

    private var bitmap:BitmapData = null;
    
    private var northHeadOffset:Array = new Array(0,2,3,3,1,-1,-3,-3,-2);
    private var eastHeadOffset:Array = new Array(2,3,3,2,-1,-2,-3,-2,-1);
    private var southHeadOffset:Array = new Array(2,3,3,2,-1,-2,-3,-2,-1);
    private var westHeadOffset:Array = new Array(0,2,3,3,1,-1,-3,-3,-2);  
    
    public function draw(screenBuffer:BitmapData,visible:Rectangle):void
    {
      if (bitmap == null)
      {
        bitmap = new BitmapData(96,80,true,0x00000000);
        bitmap.draw(new snakeImg());
      }    
    
      for (var i:int = partPosition.length - 1; i >= 0; i--)
      {
        var pos:Point = partPosition[i];
        var direction:int = partDirection[i];
        var phase:int = partPhase[i];             
               
        if (i == 0) //head
        {        
          phase = partPhase[1];         
        
          switch (direction)
          {
          case NORTH:         
            screenBuffer.copyPixels(bitmap,
                                    new Rectangle(38,51,15,17),
                                    new Point(pos.x - 7 + northHeadOffset[phase],visible.y - pos.y - 13)); 
            break;
          case EAST:
            screenBuffer.copyPixels(bitmap,
                                    new Rectangle(23,38,17,15),
                                    new Point(pos.x - 4,visible.y - pos.y - 7 + eastHeadOffset[phase])); 
            break;
          case SOUTH:         
            screenBuffer.copyPixels(bitmap,
                                    new Rectangle(38,22,15,17),
                                    new Point(pos.x - 7 + southHeadOffset[phase],visible.y - pos.y - 4)); 
            break;
          case WEST:
            screenBuffer.copyPixels(bitmap,
                                    new Rectangle(52,38,17,15),
                                    new Point(pos.x - 13,visible.y - pos.y - 7 + westHeadOffset[phase])); 
            break;
          }                    
        }
        else
        if (i == partPosition.length - 1) //tail
        {        
          switch (direction)
          {
          case NORTH:         
            screenBuffer.copyPixels(bitmap,
                                    new Rectangle(80,2,11,10),
                                    new Point(pos.x - 5 + northHeadOffset[phase],visible.y - pos.y - 3)); 
            break;
          case EAST: 
            screenBuffer.copyPixels(bitmap,
                                    new Rectangle(80,24,10,11),
                                    new Point(pos.x - 6,visible.y - pos.y - 5 + eastHeadOffset[phase]));
            break;
          case SOUTH: 
            screenBuffer.copyPixels(bitmap,
                                    new Rectangle(80,13,11,10),
                                    new Point(pos.x - 5 + southHeadOffset[phase],visible.y - pos.y - 7)); 
            break;
          case WEST:
            screenBuffer.copyPixels(bitmap,
                                    new Rectangle(80,35,10,11),
                                    new Point(pos.x - 5,visible.y - pos.y - 5 + westHeadOffset[phase]));
            break;
          }                    
        }        
        else  //body
        {
          if ((direction == EAST) || (direction == WEST))
            screenBuffer.copyPixels(bitmap,
                                    new Rectangle(18 + phase * 4,0,4,19),
                                    new Point(pos.x - 2,visible.y - pos.y - 9));
          else          
            screenBuffer.copyPixels(bitmap,
                                    new Rectangle(0,18 + phase * 4,19,4),
                                    new Point(pos.x - 9,visible.y - pos.y - 2));
        }
        
        var prevDirection:int = i < partPosition.length - 1 ? partDirection[i + 1] : direction;
        
        if (prevDirection != direction) 
          switch (prevDirection)
          {
          case NORTH:
            if (direction == EAST)
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(72,51,10,11),
                                      new Point(pos.x - 12,visible.y - pos.y - 5));             
            else
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(82,51,10,11),
                                      new Point(pos.x + 2,visible.y - pos.y - 3));             
            break;
          case EAST:
            if (direction == NORTH)
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(82,62,10,11),
                                      new Point(pos.x - 2,visible.y - pos.y + 2));             
            else
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(82,51,10,11),
                                      new Point(pos.x - 4,visible.y - pos.y - 12));            
            break;
          case SOUTH:
            if (direction == EAST)
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(72,62,10,11),
                                      new Point(pos.x - 11,visible.y - pos.y - 5));             
            else
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(82,62,10,11),
                                      new Point(pos.x + 2,visible.y - pos.y - 3));             
            break;
          case WEST:
            if (direction == NORTH)
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(72,62,10,11),
                                      new Point(pos.x - 3,visible.y - pos.y + 2));             
            else
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(72,51,10,11),
                                      new Point(pos.x - 5,visible.y - pos.y - 13));            
            break;
          }  
      } 
    }
  } 
}
