private static const GAME_FADE_IN:int        = 1;
private static const GAME_START:int          = 2;
private static const GAME_STARTED:int        = 3;
private static const GAME_PLAYED:int         = 4;

private var gameState:int = GAME_FADE_IN;
private var newState:int = 0;
private var gameTime:Number = 0;
private var frameCount:int = 0;

private var level:int = 0;

private var snake:Snake = null;

private function New_Game():void
{  
  level = 1;
  snake = new Snake(SCREEN_WIDTH / 2,SCREEN_HEIGHT / 3,20);  

  ChangeState_Game(GAME_FADE_IN);
}

private function ChangeState_Game(newState:int):void
{  
  gameState = newState;
  gameTime = 0;
}

private function Draw_Game(ellapsedTime:Number):void
{  
  gameTime += ellapsedTime;
  frameCount++;    
  
  screenBuffer.fillRect(new Rectangle(0,0,SCREEN_WIDTH,SCREEN_HEIGHT),0xFF406020);
  
  if (snake != null)
    snake.draw(screenBuffer,new Rectangle(0,SCREEN_HEIGHT,SCREEN_WIDTH,SCREEN_HEIGHT));
  
  switch (gameState)
  {
  case GAME_FADE_IN:  
    Draw_StageSign(gameTime,gameTime);
  
    Apply_FadeEffect(1 - gameTime); 
    
    if (gameTime >= 1)
      ChangeState_Game(GAME_START);       
    break;
  case GAME_START: 
    var fracTime:Number = gameTime - int(gameTime);
     
    Draw_StageSign(1,int(gameTime + 1) % 2 == 0 ? fracTime : 1 - fracTime); 
    
    if ((newState == GAME_STARTED) && 
        (int(gameTime + 1) % 2 == 0 ? fracTime > 0.55 : fracTime < 0.55))
    {
      ChangeState_Game(GAME_STARTED);     
      newState = 0;
    }
    break;   
  case GAME_STARTED: 
    Draw_StageSign(1 - gameTime,1 - gameTime); 
    
    if (gameTime >= 1)
      ChangeState_Game(GAME_PLAYED);       
    break;
  case GAME_PLAYED:        
    if (snake != null)
      snake.move();
    break;    
  }  
  
  SetGameCursor();  
}

private var stageLabelBitmap:BitmapData = null;
private var stageReadyBitmap:BitmapData = null;

private function Draw_StageSign(stageOpacity:Number,readyOpacity:Number):void
{
  if (stageLabelBitmap == null)
  {
    stageLabelBitmap = new BitmapData(350,160,true,0x00000000);
    stageLabelBitmap.draw(new stageLabelImg());
    
    stageReadyBitmap = new BitmapData(240,250,true,0x00000000);
    stageReadyBitmap.draw(new stageReadyImg());
  }
  
  if (stageOpacity > 0.15)   
  {
    var yPos:int = 0;
    
    if (stageOpacity < 0.25)
      yPos = 128;
    else
    if (stageOpacity < 0.45)
      yPos = 96;
    else
    if (stageOpacity < 0.45)
      yPos = 64;           
    else
    if (stageOpacity < 0.55)
      yPos = 32; 
                          
    screenBuffer.copyPixels(stageLabelBitmap,
                            new Rectangle(0, yPos, 100, 32),
                            new Point(SCREEN_WIDTH / 2 - 80,SCREEN_HEIGHT / 2 - 120));
                            
    screenBuffer.copyPixels(stageLabelBitmap,
                            new Rectangle(int(level / 10) * 24 + 102, yPos, 24, 32),
                            new Point(SCREEN_WIDTH / 2 + 30,SCREEN_HEIGHT / 2 - 120)); 
                                                        
    screenBuffer.copyPixels(stageLabelBitmap,
                            new Rectangle((level % 10) * 24 + 102, yPos, 24, 32),
                            new Point(SCREEN_WIDTH / 2 + 54,SCREEN_HEIGHT / 2 - 120)); 
  }
                                 
  if (readyOpacity > 0.15)   
  {
    var yPos2:int = 0;
    
    if (readyOpacity < 0.25)
      yPos2 = 200;
    else
    if (readyOpacity < 0.45)
      yPos2 = 150;
    else
    if (readyOpacity < 0.45)
      yPos2 = 100;           
    else
    if (readyOpacity < 0.55)
      yPos2 = 50;      
                  
    screenBuffer.copyPixels(stageReadyBitmap,
                            new Rectangle(0, yPos2, 240, 50),
                            new Point(SCREEN_WIDTH / 2 - 120,SCREEN_HEIGHT / 2 - 50));  
  }
}

private function SetGameCursor():void 	
{  
  var mx:int = mouseX;
  var my:int = mouseY;
  
  my = SCREEN_HEIGHT - my;
  var position:Point = snake.getPosition();
  var direction:int = snake.getDirection(); 
  var cursorDirection:int = 0;
  var cursorDisabled:Boolean = true;   
        
  if (Math.abs(mx - position.x) > Math.abs(my - position.y))
  { //check east/west
    if (mx - position.x > 0)  
      cursorDirection = Snake.EAST;      
    else
      cursorDirection = Snake.WEST;
      
    if (Math.abs(mx - position.x) > Snake.SPEED)      
      if ((direction == Snake.NORTH) || (direction == Snake.SOUTH)) 
        cursorDisabled = false;
  }
  else
  { //check south/north
    if (my - position.y > 0)
      cursorDirection = Snake.NORTH;      
    else
      cursorDirection = Snake.SOUTH;
      
    if (Math.abs(my - position.y) > Snake.SPEED)
      if ((direction == Snake.EAST) || (direction == Snake.WEST))  
        cursorDisabled = false;         
  }
  
  SetMouseCursor(cursorDirection,cursorDisabled);  
}

private function MouseDown_Game(event:MouseEvent):void 	
{	
  var mx:int = event.localX;
  var my:int = event.localY;
  
  switch (gameState)
  {
  case GAME_START:  
    newState = GAME_STARTED;   
    break;
  case GAME_PLAYED:
    {
      my = SCREEN_HEIGHT - my;
      var position:Point = snake.getPosition();
      var direction:int = snake.getDirection();      
        
      if ((Math.abs(mx - position.x) > Snake.SPEED) || 
          (Math.abs(my - position.y) > Snake.SPEED))
        if (Math.abs(mx - position.x) > Math.abs(my - position.y))
        { //check east/west
          if ((direction == Snake.NORTH) || (direction == Snake.SOUTH))        
            if (mx - position.x > 0)
              snake.setDirection(Snake.EAST);
            else
              snake.setDirection(Snake.WEST);
        }
        else
        { //check south/north
          if ((direction == Snake.EAST) || (direction == Snake.WEST))            
            if (my - position.y > 0)
              snake.setDirection(Snake.NORTH);
            else
              snake.setDirection(Snake.SOUTH);        
        }
    }
    break;    
  } 
}	
	
private function MouseUp_Game(event:MouseEvent):void 	
{	

}

private function MouseMoved_Game(event:MouseEvent):void 	
{

}