private static const TITLE_SHOW:int       = 1;
private static const GAMEOPTIONS_IN:int   = 2;
private static const GAMEOPTIONS_WAIT:int = 3;
private static const GAMEOPTIONS_OUT:int  = 4;
private static const TITLE_HIDE:int       = 5;

private var gameMenuState:int = TITLE_SHOW;
private var gameMenuTime:Number = 0;

private static const OPTION_NEW_GAME:int  = 0;
private static const OPTION_CONTINUE:int  = 1;
private static const OPTION_SETTINGS:int  = 2;

private var pressedOption:int = -1;
private var selectedOption:int = -1;

private function Draw_GameMenu(ellapsedTime:Number):void
{ 
  gameMenuTime += ellapsedTime;
   
  switch (gameMenuState)
  {
  case TITLE_SHOW:
    Draw_TitleScreen(); 
    
    Apply_FadeEffect(1 - gameMenuTime); 
        
    if (gameMenuTime >= 1)
      ChangeState_GameMenu(GAMEOPTIONS_IN);     
    break;
  case GAMEOPTIONS_IN:
    Draw_TitleScreen();
    Draw_GameOptions();
    
    if (gameMenuTime >= 1)
      ChangeState_GameMenu(GAMEOPTIONS_WAIT); 
    break;   
  case GAMEOPTIONS_WAIT: 
    Draw_TitleScreen();
    Draw_GameOptions();
    break;   
  case GAMEOPTIONS_OUT:  
    Draw_TitleScreen();
    Draw_GameOptions();
    
    if (gameMenuTime >= 1)
      ChangeState_GameMenu(TITLE_HIDE);
    break;    
  case TITLE_HIDE:
    Draw_TitleScreen();
    
    Apply_FadeEffect(gameMenuTime);
    
    if (gameMenuTime >= 1)
      switch (selectedOption)
      {
      case OPTION_NEW_GAME:
        state = GAME;
        New_Game();
        break;       
      }
    break;  
  }
}

private function ChangeState_GameMenu(newState:int):void
{
  switch(newState)
  {
  case GAMEOPTIONS_IN: 
    PlaySound(OPTIONSIN_SOUND);  
    break;
  case GAMEOPTIONS_OUT:
    PlaySound(OPTIONSOUT_SOUND);  
    break;
  }
  
  gameMenuState = newState;
  gameMenuTime = 0;
}

private var titleScreenBitmap:BitmapData = null;

private function Draw_TitleScreen():void
{
  if (titleScreenBitmap == null)
  {
    titleScreenBitmap = new BitmapData(SCREEN_WIDTH,SCREEN_HEIGHT,true);
    titleScreenBitmap.draw(new titleScreenImg());
  }

  screenBuffer.copyPixels(titleScreenBitmap,
                          new Rectangle(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT),
                          new Point(0,0));
}

private var gameOptionsBitmap:BitmapData = null;
private var gameOptionBackgroundBitmap:BitmapData = null;

private function Draw_GameOptions():void
{                          
  if (gameOptionsBitmap == null)
  {   
    gameOptionsBitmap = new BitmapData(260,200,true,0x00000000);
    gameOptionsBitmap.draw(new gameOptionsImg()); 
  }   
    
  if (gameOptionBackgroundBitmap == null)
  {    
    gameOptionBackgroundBitmap = new BitmapData(340,240,true,0x00000000);
    gameOptionBackgroundBitmap.draw(new gameOptionBackgroundImg());    
  }
  
  for (var i:int = OPTION_NEW_GAME; i <= OPTION_SETTINGS; i++)
  { 
    var srcY:int = pressedOption == i ? 160 : 80;
    
    if (i == OPTION_CONTINUE)
      srcY = 0;
      
    var srcX:int = 0;
    
    if (gameMenuState == GAMEOPTIONS_IN) 
    {
      srcX = -((1 - gameMenuTime) * 3 - Number((OPTION_SETTINGS - i) / 2)) * 400;
      
      if (srcX > 0)
        srcX = 0;      
    }
      
    if (gameMenuState == GAMEOPTIONS_OUT)
    {
      srcX = (gameMenuTime * 3 - Number(i / 2)) * 400;
      
      if (srcX < 0)
        srcX = 0;   
    }  
  
    screenBuffer.copyPixels(gameOptionBackgroundBitmap,
                            new Rectangle(0, srcY, 340, 80),
                            new Point(70 + srcX,235 + i * 70 + (pressedOption == i ? 1 : 0)));
    
    switch (i)
    {
    case OPTION_NEW_GAME:
      srcY = 0;
      break;
    case OPTION_CONTINUE:
      srcY = 100;
      break;
    case OPTION_SETTINGS:
      srcY = 150;
      break;
    }
    
    screenBuffer.copyPixels(gameOptionsBitmap,
                            new Rectangle(0, srcY, 260, 50),
                            new Point(110 + srcX,250 + i * 70 + (pressedOption == i ? 1 : 0)));
  }
}

private function MouseDown_GameMenu(event:MouseEvent):void 	
{	
	var mx:int = event.localX;
  var my:int = event.localY;
	  
	if (gameMenuState == GAMEOPTIONS_WAIT)
    for (var i:int = OPTION_NEW_GAME; i <= OPTION_SETTINGS; i++)	 
      if (i != OPTION_CONTINUE)
        if ((mx >= 110) && (mx < 370) && (my >= 250 + i * 70) && (my < 300 + i * 70)) 
        {	  	 
          pressedOption = i;
          PlaySound(OPTIONSELECTED_SOUND);          
        }
}	
	
private function MouseUp_GameMenu(event:MouseEvent):void 	
{	
  var mx:int = event.localX;
  var my:int = event.localY;
  
  switch (gameMenuState)
  {
  case GAMEOPTIONS_WAIT:  
    switch (pressedOption)
    {
    case OPTION_NEW_GAME:		  
      selectedOption = OPTION_NEW_GAME;      
      ChangeState_GameMenu(GAMEOPTIONS_OUT);      
      break;
    }
    break;
  }  
  
  pressedOption = -1;
}

private function MouseMoved_GameMenu(event:MouseEvent):void 	
{
  var mx:int = event.localX;
  var my:int = event.localY;
  
  for (var i:int = OPTION_NEW_GAME; i <= OPTION_SETTINGS; i++)
    if (pressedOption == i)
      if ((mx < 110) || (mx > 370) || (my < 250 + i * 70) || (my > 300 + i * 70)) 	  	 
        pressedOption = -1; 
}