import flash.display.*; 
import flash.events.*;
import flash.external.ExternalInterface; 
import mx.events.*;
import mx.controls.*;
import mx.managers.CursorManager;
import Classes.*;

include "Images.as";
include "Sounds.as";
include "Effects.as";

include "Activation.as";
include "GameMenu.as";
include "Game.as";

private static const SCREEN_WIDTH:int     = 480;
private static const SCREEN_HEIGHT:int    = 480;

private static const ACTIVATION:int       = 0;
private static const GAME_MENU:int        = 1;
private static const GAME:int             = 2;

private var state:int = GAME_MENU;

private var initializationCompleted:Boolean = false;
private var screenBuffer:BitmapData = null;

public function Init():void
{  	
  SetMouseCursor(0,false);

  screenBuffer = new BitmapData(SCREEN_WIDTH, SCREEN_HEIGHT, false, 0x00000000);
  
	if (ExternalInterface.available)
    state = ACTIVATION; 
  
  initializationCompleted = true; 
}

private var prevTime:Date = null;

private function UpdateFrame():void
{
  if (!initializationCompleted)
    return;

  var currTime:Date = new Date();
  var ellapsedTime:Number = 0;
  
  if (prevTime != null) 
    ellapsedTime = (currTime.getTime() - prevTime.getTime()) / 1000.0;
    
  prevTime = currTime;  
  
  Draw(ellapsedTime);
    
  gamePanel.graphics.clear();    
  gamePanel.graphics.beginBitmapFill(screenBuffer, null, false, false);
  gamePanel.graphics.drawRect(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
  gamePanel.graphics.endFill();
}

private function Draw(ellapsedTime:Number):void
{
  switch (state)
  {
  case ACTIVATION:
    Draw_Activation(ellapsedTime);
    break;  
  case GAME_MENU:
    Draw_GameMenu(ellapsedTime);
    break;
  case GAME:
    Draw_Game(ellapsedTime);  
    break;   
  }
}

private var cursorID:Number = 0;
private var mouseCursorDirection:int = -1;
private var mouseCursorDisabled:Boolean = false;
	
private function SetMouseCursor(direction:int,disabled:Boolean):void 	
{
  if ((mouseCursorDirection != direction) || (mouseCursorDisabled != disabled))
  {
    mouseCursorDirection = direction;
    mouseCursorDisabled = disabled;
  
		CursorManager.removeCursor(cursorID);	  
	  
	  switch (direction)
	  {
	  case 0:
      cursorID = CursorManager.setCursor(mouseCursorImg);
      break;
	  case Snake.NORTH:
      if (disabled)
        cursorID = CursorManager.setCursor(mouseCursorNDImg);
      else
        cursorID = CursorManager.setCursor(mouseCursorNImg);      
      break; 
	  case Snake.EAST:
      if (disabled)
        cursorID = CursorManager.setCursor(mouseCursorEDImg);
      else
        cursorID = CursorManager.setCursor(mouseCursorEImg);      
      break; 
	  case Snake.SOUTH:
      if (disabled)
        cursorID = CursorManager.setCursor(mouseCursorSDImg);
      else
        cursorID = CursorManager.setCursor(mouseCursorSImg);      
      break; 
	  case Snake.WEST:
      if (disabled)
        cursorID = CursorManager.setCursor(mouseCursorWDImg);
      else
        cursorID = CursorManager.setCursor(mouseCursorWImg);      
      break;   
    }
	}
}

private function MouseDown(event:MouseEvent):void 	
{	
  switch (state)
  {
  case ACTIVATION:
    MouseDown_Activation(event);
    break;  
  case GAME_MENU:
    MouseDown_GameMenu(event);
    break;
  case GAME:
    MouseDown_Game(event);
    break;
  }
}	
	
private function MouseUp(event:MouseEvent):void 	
{	
  switch (state)
  {
  case ACTIVATION:
    MouseUp_Activation(event);
    break;  
  case GAME_MENU:
    MouseUp_GameMenu(event);
    break;
  case GAME:
    MouseUp_Game(event);
    break;
  }
}

private function MouseMoved(event:MouseEvent):void 	
{
  switch (state)
  {
  case GAME_MENU:
    MouseMoved_GameMenu(event);
    break;
  case GAME:
    MouseMoved_Game(event);
    break;
  }
}

